"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.average = void 0;
const tslib_1 = require("tslib");
const identity_js_1 = require("../util/identity.js");
const withabort_js_1 = require("./operators/withabort.js");
const aborterror_js_1 = require("../aborterror.js");
/**
 * Computes the average of the async-iterable sequence.
 *
 * @param {AsyncIterable<any>} source source async-iterable sequence to compute the average.
 * @param {AverageOptions<any>} [options] The options for calculating the average.
 * @returns {Promise<number>} A Promise which returns the computed average for the async-iterable sequence.
 */
function average(source, options) {
    var _a, e_1, _b, _c;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { ['selector']: selector = identity_js_1.identityAsync, ['signal']: signal, ['thisArg']: thisArg, } = options || {};
        (0, aborterror_js_1.throwIfAborted)(signal);
        let sum = 0;
        let count = 0;
        try {
            for (var _d = true, _e = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(source, signal)), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                _c = _f.value;
                _d = false;
                const item = _c;
                sum += yield selector.call(thisArg, item, signal);
                count++;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (count === 0) {
            throw new Error('Empty collection');
        }
        return sum / count;
    });
}
exports.average = average;

//# sourceMappingURL=average.js.map
