"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.count = void 0;
const tslib_1 = require("tslib");
const withabort_js_1 = require("./operators/withabort.js");
const aborterror_js_1 = require("../aborterror.js");
/**
 * Returns a promise that represents how many elements in the specified async-iterable sequence satisfy a condition
 * otherwise, the number of items in the sequence.
 *
 * @template T The type of elements in the source collection.
 * @param {AsyncIterable<T>} source An async-iterable sequence that contains elements to be counted.
 * @param {OptionalFindOptions<T>} [options] The options for a predicate for filtering, thisArg for binding and AbortSignal for cancellation.
 * @returns {Promise<number>} The number of matching elements for the given condition if provided, otherwise
 * the number of elements in the sequence.
 */
function count(source, options) {
    var _a, e_1, _b, _c;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { ['signal']: signal, ['thisArg']: thisArg, ['predicate']: predicate = () => tslib_1.__awaiter(this, void 0, void 0, function* () { return true; }) } = options || {};
        (0, aborterror_js_1.throwIfAborted)(signal);
        let i = 0;
        try {
            for (var _d = true, _e = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(source, signal)), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                _c = _f.value;
                _d = false;
                const item = _c;
                if (yield predicate.call(thisArg, item, i, signal)) {
                    i++;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return i;
    });
}
exports.count = count;

//# sourceMappingURL=count.js.map
