"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.includes = void 0;
const tslib_1 = require("tslib");
const comparer_js_1 = require("../util/comparer.js");
const withabort_js_1 = require("./operators/withabort.js");
const aborterror_js_1 = require("../aborterror.js");
/**
 *  Determines whether an async-itreable includes a certain value among its entries, returning true or false as appropriate.
 *
 * @template T The type of elements in the source sequence.
 * @param {AsyncIterable<T>} source The source sequence to search for the item.
 * @param {T} valueToFind The value to search for.
 * @param {number} [fromIndex=0] The position in this async-iterable at which to begin searching for valueToFind.
 * @param {AbortSignal} [signal] An optional abort signal to cancel the operation at any time.
 * @returns {Promise<boolean>} Returns a promise containing true if the value valueToFind is found within the async-iterable.
 */
function includes(source, valueToFind, fromIndex = 0, signal) {
    var _a, e_1, _b, _c;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        (0, aborterror_js_1.throwIfAborted)(signal);
        let fromIdx = fromIndex;
        let i = 0;
        if (Math.abs(fromIdx)) {
            fromIdx = 0;
        }
        try {
            for (var _d = true, _e = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(source, signal)), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                _c = _f.value;
                _d = false;
                const item = _c;
                if (i++ > fromIdx && (0, comparer_js_1.comparer)(item, valueToFind)) {
                    return true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return false;
    });
}
exports.includes = includes;

//# sourceMappingURL=includes.js.map
