"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.merge = exports.MergeAsyncIterable = void 0;
const tslib_1 = require("tslib");
const asynciterablex_js_1 = require("./asynciterablex.js");
const withabort_js_1 = require("./operators/withabort.js");
const aborterror_js_1 = require("../aborterror.js");
const safeRace_js_1 = require("../util/safeRace.js");
// eslint-disable-next-line @typescript-eslint/no-empty-function
const NEVER_PROMISE = new Promise(() => { });
function wrapPromiseWithIndex(promise, index) {
    return promise
        .then(({ value, done }) => ({ value, done, index }))
        .catch((error) => ({ error, index }));
}
/** @ignore */
class MergeAsyncIterable extends asynciterablex_js_1.AsyncIterableX {
    constructor(source) {
        super();
        this._source = source;
    }
    [Symbol.asyncIterator](signal) {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            (0, aborterror_js_1.throwIfAborted)(signal);
            const length = this._source.length;
            const iterators = new Array(length);
            const nexts = new Array(length);
            let active = length;
            for (let i = 0; i < length; i++) {
                const iterator = (0, withabort_js_1.wrapWithAbort)(this._source[i], signal)[Symbol.asyncIterator]();
                iterators[i] = iterator;
                nexts[i] = wrapPromiseWithIndex(iterator.next(), i);
            }
            while (active > 0) {
                const next = yield tslib_1.__await((0, safeRace_js_1.safeRace)(nexts));
                if (next.hasOwnProperty('error')) {
                    throw next.error;
                }
                else if (next.done) {
                    nexts[next.index] = NEVER_PROMISE;
                    active--;
                }
                else {
                    const iterator$ = iterators[next.index];
                    nexts[next.index] = wrapPromiseWithIndex(iterator$.next(), next.index);
                    yield yield tslib_1.__await(next.value);
                }
            }
        });
    }
}
exports.MergeAsyncIterable = MergeAsyncIterable;
function merge(source, ...args) {
    return new MergeAsyncIterable([source, ...args]);
}
exports.merge = merge;

//# sourceMappingURL=merge.js.map
