"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.filter = exports.FilterAsyncIterable = void 0;
const tslib_1 = require("tslib");
const asynciterablex_js_1 = require("../asynciterablex.js");
const withabort_js_1 = require("./withabort.js");
const aborterror_js_1 = require("../../aborterror.js");
/** @ignore */
class FilterAsyncIterable extends asynciterablex_js_1.AsyncIterableX {
    constructor(source, predicate, thisArg) {
        super();
        this._source = source;
        this._predicate = predicate;
        this._thisArg = thisArg;
    }
    [Symbol.asyncIterator](signal) {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var _b, e_1, _c, _d;
            (0, aborterror_js_1.throwIfAborted)(signal);
            let i = 0;
            try {
                for (var _e = true, _f = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(this._source, signal)), _g; _g = yield tslib_1.__await(_f.next()), _b = _g.done, !_b; _e = true) {
                    _d = _g.value;
                    _e = false;
                    const item = _d;
                    if (yield tslib_1.__await(this._predicate.call(this._thisArg, item, i++))) {
                        yield yield tslib_1.__await(item);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_e && !_b && (_c = _f.return)) yield tslib_1.__await(_c.call(_f));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
}
exports.FilterAsyncIterable = FilterAsyncIterable;
/**
 * Filters the elements of an async-iterable sequence based on a predicate.
 *
 * @template TSource The type of the elements in the source sequence.
 * @param {((value: TSource, index: number, signal?: AbortSignal) => boolean | Promise<boolean>)} predicate A function to test each source element
 * for a condition.
 * @param {*} [thisArg] Optional this for binding.
 * @returns {OperatorAsyncFunction<TSource, TSource>} An operator which returns an async-iterable
 * sequence that contains elements from the input sequence that satisfy the condition.
 */
function filter(predicate, thisArg) {
    return function filterOperatorFunction(source) {
        return new FilterAsyncIterable(source, predicate, thisArg);
    };
}
exports.filter = filter;

//# sourceMappingURL=filter.js.map
