"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.finalize = exports.FinallyAsyncIterable = void 0;
const tslib_1 = require("tslib");
const asynciterablex_js_1 = require("../asynciterablex.js");
const withabort_js_1 = require("./withabort.js");
const aborterror_js_1 = require("../../aborterror.js");
/** @ignore */
class FinallyAsyncIterable extends asynciterablex_js_1.AsyncIterableX {
    constructor(source, action) {
        super();
        this._source = source;
        this._action = action;
    }
    [Symbol.asyncIterator](signal) {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var _b, e_1, _c, _d;
            (0, aborterror_js_1.throwIfAborted)(signal);
            try {
                try {
                    for (var _e = true, _f = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(this._source, signal)), _g; _g = yield tslib_1.__await(_f.next()), _b = _g.done, !_b; _e = true) {
                        _d = _g.value;
                        _e = false;
                        const item = _d;
                        yield yield tslib_1.__await(item);
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (!_e && !_b && (_c = _f.return)) yield tslib_1.__await(_c.call(_f));
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            }
            finally {
                yield tslib_1.__await(this._action());
            }
        });
    }
}
exports.FinallyAsyncIterable = FinallyAsyncIterable;
/**
 *  Invokes a specified asynchronous action after the source async-iterable sequence terminates gracefully or exceptionally.
 *
 * @template TSource The type of the elements in the source sequence.
 * @param {(() => void | Promise<void>)} action Action to invoke and await asynchronously after the source async-iterable sequence terminates
 * @returns {MonoTypeOperatorAsyncFunction<TSource>} An operator that returns the source sequence with the
 * action-invoking termination behavior applied.
 */
function finalize(action) {
    return function finalizeOperatorFunction(source) {
        return new FinallyAsyncIterable(source, action);
    };
}
exports.finalize = finalize;

//# sourceMappingURL=finalize.js.map
