"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.share = void 0;
const tslib_1 = require("tslib");
const asynciterablex_js_1 = require("../asynciterablex.js");
const create_js_1 = require("../create.js");
const aborterror_js_1 = require("../../aborterror.js");
class SharedAsyncIterable extends asynciterablex_js_1.AsyncIterableX {
    constructor(it) {
        super();
        this._it = {
            next(value) {
                return it.next(value);
            },
        };
    }
    [Symbol.asyncIterator](signal) {
        (0, aborterror_js_1.throwIfAborted)(signal);
        return this._it;
    }
}
/**
 * Shares the source sequence within a selector function where each iterator can fetch the next element from the
 * source sequence.
 *
 * @template TSource Source sequence element type.
 * @template TResult Result sequence element type.
 * @param {((
 *     value: AsyncIterable<TSource>,
 *     signal?: AbortSignal
 *   ) => AsyncIterable<TResult> | Promise<AsyncIterable<TResult>>)} [selector] Selector function with shared access
 * to the source sequence for each iterator.
 * @returns {(OperatorAsyncFunction<TSource, TSource | TResult>)} Sequence resulting from applying the selector function to the
 * shared view over the source sequence.
 */
function share(selector) {
    return function shareOperatorFunction(source) {
        return selector
            ? (0, create_js_1.create)((signal) => tslib_1.__awaiter(this, void 0, void 0, function* () {
                const it = yield selector(new SharedAsyncIterable(source[Symbol.asyncIterator](signal)), signal);
                return it[Symbol.asyncIterator](signal);
            }))
            : new SharedAsyncIterable(source[Symbol.asyncIterator]());
    };
}
exports.share = share;

//# sourceMappingURL=share.js.map
