"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.throttle = exports.ThrottleAsyncIterable = void 0;
const tslib_1 = require("tslib");
const asynciterablex_js_1 = require("../asynciterablex.js");
const withabort_js_1 = require("./withabort.js");
const aborterror_js_1 = require("../../aborterror.js");
/** @ignore */
class ThrottleAsyncIterable extends asynciterablex_js_1.AsyncIterableX {
    constructor(source, time) {
        super();
        this._source = source;
        this._time = time;
    }
    [Symbol.asyncIterator](signal) {
        return tslib_1.__asyncGenerator(this, arguments, function* _a() {
            var _b, e_1, _c, _d;
            (0, aborterror_js_1.throwIfAborted)(signal);
            let currentTime;
            let previousTime;
            try {
                for (var _e = true, _f = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(this._source, signal)), _g; _g = yield tslib_1.__await(_f.next()), _b = _g.done, !_b; _e = true) {
                    _d = _g.value;
                    _e = false;
                    const item = _d;
                    currentTime = Date.now();
                    if (!previousTime || currentTime - previousTime > this._time) {
                        previousTime = currentTime;
                        yield yield tslib_1.__await(item);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (!_e && !_b && (_c = _f.return)) yield tslib_1.__await(_c.call(_f));
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
}
exports.ThrottleAsyncIterable = ThrottleAsyncIterable;
/**
 * Throttles the source async-iterable sequence so that it doesn't emit more than one value during the given timeframe.
 *
 * @template TSource The type of elements in the source sequence.
 * @param {number} time The time in milliseconds to throttle the source sequence.
 * @returns {MonoTypeOperatorAsyncFunction<TSource>} The source sequence throttled by the given timeframe.
 */
function throttle(time) {
    return function throttleOperatorFunction(source) {
        return new ThrottleAsyncIterable(source, time);
    };
}
exports.throttle = throttle;

//# sourceMappingURL=throttle.js.map
