"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.single = void 0;
const tslib_1 = require("tslib");
const withabort_js_1 = require("./operators/withabort.js");
const aborterror_js_1 = require("../aborterror.js");
/**
 * Returns the only element of an async-iterable sequence that matches the predicate if specified,
 * or undefined if no such element exists; this method reports an exception if there is more
 * than one element in the async-iterable sequence.
 *
 * @template T The type of the elements in the source sequence.
 * @param {AsyncIterable<T>} source Source async-iterable sequence.
 * @param {OptionalFindOptions<T>} [options] The optional options which includes a predicate for filtering,
 * thisArg for predicate binding and an abort signal for cancellation.
 * @returns {(Promise<T | undefined>)} A promise with the single element in the async-iterable sequence that satisfies
 * the condition in the predicate, or undefined if no such element exists.
 */
function single(source, options) {
    var _a, e_1, _b, _c;
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const { ['signal']: signal, ['thisArg']: thisArg, ['predicate']: predicate = () => tslib_1.__awaiter(this, void 0, void 0, function* () { return true; }) } = options || {};
        (0, aborterror_js_1.throwIfAborted)(signal);
        let result;
        let hasResult = false;
        let i = 0;
        try {
            for (var _d = true, _e = tslib_1.__asyncValues((0, withabort_js_1.wrapWithAbort)(source, signal)), _f; _f = yield _e.next(), _a = _f.done, !_a; _d = true) {
                _c = _f.value;
                _d = false;
                const item = _c;
                if (hasResult && (yield predicate.call(thisArg, item, i++, signal))) {
                    throw new Error('More than one element was found');
                }
                if (yield predicate.call(thisArg, item, i++, signal)) {
                    result = item;
                    hasResult = true;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (!_d && !_a && (_b = _e.return)) yield _b.call(_e);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return result;
    });
}
exports.single = single;

//# sourceMappingURL=single.js.map
